/* ping-scanner-call-multiple.c          Version 1.00  2007 February 19
 *
 * Compile with the GCC C compiler:
 * 
 *   gcc ping-scanner-call-multiple.c -o ping-scanner-call-multiple
 *
 * This program fires up a set of ping-scanner programs, each on a 
 * different subnet, in parallel.  
 *
 * Copyright public domain - Robin Whittle hhtp://www.firstpr.com.au/ip/
 *
 * Please use this judiciously.  Folks may be upset about being pinged, 
 * especially if they are the US Department of Defence!
 *
 * Also, don't run too many ping-scanner programs in parallel since it 
 * could choke up your computer or link to the Net.  I ran them 50 at a time,
 * such as:
 *   
 *    ping-scanner-call-multiple 50 99 1000
 *  
 */

#include <stdio.h>

main (int argc, char *argv[])
{

    long unsigned int loopcount;
        
                                    /* Starting and ending prefixes.
                                     * Can't ping prefix 0.
                                     */
    int prefix_start = 203;
    int prefix_end =   205;

                                    /* Number of pings to send to each.
                                     */
    int pingnumber = 10;
                                         
                                    /* String for command line.
                                     */
    char commandline[80];




    if (argc < 4)
    {
        printf("Error:\n");
        printf("There must be at least three numeric arguments.\n");
        printf("                                               \n");
        printf("  ping-scanner-call-multiple ppp qqq nnnnn\n");
        printf("                                               \n");
        printf("   ppp is 1 to 255 for the subnet number to start with.\n");
        printf("   qqq is 1 to 255 for the subnet number to end with.\n");
        printf("   nnnnn is 1 to 86400 for the number of pings to send.\n\n");
        exit(1);
    }


                                    /* Process the command line arguments.
                                     */
                                         
    prefix_start = atoi(argv[1]);                                        
    prefix_end = atoi(argv[2]);                                      
    pingnumber = atoi(argv[3]);                                      
    
    if (   (prefix_start < 1)
         ||(prefix_start > 255)
       )
    {
        printf("\nppp out of range = %d.\n\n", prefix_start);
        exit(1);
    }     
    
    if (   (prefix_end < 1)
         ||(prefix_end > 255)
         ||(prefix_end < prefix_start) 
       )
    {
        printf("\nqqq out of range = %d.\n\n", prefix_end);
        exit(1);
    }     
    
    if (   (pingnumber < 1)
         ||(pingnumber > 86400)
       )
    {
        printf("\nnnnnn out of range = %d.\n", pingnumber);
        exit(1);
    }   

    
    for(loopcount = prefix_start; loopcount <= prefix_end; loopcount++)
    {
        sprintf(commandline, "./ping-scanner %d %d &\n", loopcount, pingnumber );

        printf(">>>>>>>>> %s \n", &commandline );   
        
                                    /* Run the instance of the ping-scanner in the
                                     * background, with "&" at the end of the line.
                                     *
                                     * Comment this out to test it.  The
                                     * ping-scanner program is hard to kill manually.
                                     */  
        system(&commandline);

                                    /* Sleep for 2.004msec  moment so I can see
                                     * the first line of their output, I think, and
                                     * to scatter the starting times of the child 
                                     * processes so they don't all line up with one 
                                     * second.
                                     */
        system("sleep 2.004");                                   
    
    }
    
    return 0;
}
