/* ping-scanner-slash24.c          Version 1.00  2007 February 22
 *
 * This program pings 256 IP addresses in a particular IPv4 /24 prefix, such as 203.12.5.0/24.
 * It counts the number of responses, which arrive in less than a second, and writes
 * the total to a text file with the same name as the prefix.
 *
 *
 * Compile with the GCC C compiler:
 * 
 *   gcc ping-scanner-slash24.c -o ping-scanner-slash24
 *
 * Check the -w or -t option of your ping program - the one which enables it to exit
 * after a certain time.  Some are -w and some are -t.  Below, I assume -w.
 *
 * Copyright public domain - Robin Whittle hhtp://www.firstpr.com.au/ip/
 *
 * Please use this judiciously.  Folks may be upset about being pinged, 
 * especially if they are the US Department of Defence!
 *
 */

#include <stdio.h>
#include <string.h>

main (int argc, char *argv[])
{

    long unsigned int loopcount;
                                    /* Bytes 3, 2, 1 and 0 of address to ping.
                                     */
    unsigned int a3, a2, a1, a0;
    
                                    /* Number of pings returned so far.
                                     */
    
    long int pingcount = 0;
                                    /* Percentage of pings returned.
                                     */
    float percentage = 0;    
                                    /* String to prepare the command line in.
                                     */
    char commandline[80];
    
                                    /* String to prepare the output filen name in.
                                     */
    char outfilename[80];

    FILE *outfile;                  /* Output file buffer.
                                     */ 

                                    /* How many pings to send.
                                     */
    int looplength = 256;   




    if (argc < 3)
    {
        printf("Error:\n");
        printf("There must be three numeric arguments.\n");
        printf(" ping-scanner-slash24 aaa bbb ccc\n");
        printf("                                             \n");
        printf("   This is to scan the /24 prefix aaa.bbb.ccc.0\n");
        exit(1);
    }

                                    /* Process the command line arguments.
                                     */
                                         
    a3 =         atoi(argv[1]);                                      
    a2 =         atoi(argv[2]);                                      
    a1 =         atoi(argv[3]);                                      

    
    if (   (a3 < 1)
         ||(a3 > 255)
       )
    {
        printf("\naaa out of range (1 to 255) = %d.\n\n", a3);
        exit(1);
    }     
    
    if (   (a2 < 0)
         ||(a2 > 255)
       )
    {
        printf("\nbbb out of range (0 to 255) = %d.\n\n", a2);
        exit(1);
    }     

    if (   (a1 < 0)
         ||(a1 > 255)
       )
    {
        printf("\nccc out of range (0 to 255) = %d.\n\n", a2);
        exit(1);
    }     

                                    /* Open output file with name such as "013.txt"
                                     */
    sprintf(outfilename, "%d.%d.%d.0.txt", a3, a2, a1);
    if ( ( outfile = fopen(outfilename,"w") ) == NULL)
    {
        printf("Can't open %s output file\n",outfile);
        exit(1);
    }
    
    
    printf("\nLets Go!  Pinging the whole prefix %d.%d.%d.0 Output filename is %s\n", a3, a2, a1, &outfilename);
    

    
    for(loopcount = 0; loopcount < (unsigned)looplength; loopcount++)
    {
    
        sprintf(commandline, "ping -w 1 %d.%d.%d.%d\n", a3, a2, a1, loopcount );

        if (system(commandline) == 0)
        {
            pingcount++;
        }
        
        printf("Run %d, packets recieved = %d\n", loopcount, pingcount );   
            
    }
                                    /* Print results: prefix number, number
                                     * of response packets we got and the
                                     * percentage this represents.
                                     */ 
        
        percentage = 100 * (float)pingcount / (float)looplength;

        fprintf(outfile, "%03d.%03d.%03d.0,  %05d,  %8.4f\n", a3, a2, a1, pingcount, percentage);                               
    
    return 0;
}
